@@ -49,21 +49,18 @@
 
 package com.openexchange.report.client.impl;
 
-import java.io.IOException;
 import java.lang.reflect.Field;
+import java.text.SimpleDateFormat;
 import java.util.Collections;
 import java.util.Date;
 import java.util.HashMap;
+import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
-import javax.management.AttributeNotFoundException;
-import javax.management.InstanceNotFoundException;
 import javax.management.InvalidAttributeValueException;
-import javax.management.MBeanException;
 import javax.management.MBeanServerConnection;
 import javax.management.MalformedObjectNameException;
 import javax.management.ObjectName;
-import javax.management.ReflectionException;
 import javax.management.openmbean.CompositeData;
 import org.json.JSONException;
 import org.json.JSONObject;
@@ -72,6 +69,7 @@ import com.openexchange.admin.console.AdminParser;
 import com.openexchange.admin.console.AdminParser.NeededQuadState;
 import com.openexchange.admin.console.CLIOption;
 import com.openexchange.groupware.userconfiguration.UserConfiguration;
+import com.openexchange.report.appsuite.serialization.ReportConfigs;
 import com.openexchange.report.client.container.ClientLoginCount;
 import com.openexchange.report.client.container.ContextDetail;
 import com.openexchange.report.client.container.MacDetail;
@@ -82,6 +80,12 @@ import com.openexchange.tools.console.TableWriter;
 import com.openexchange.tools.console.TableWriter.ColumnFormat;
 import com.openexchange.tools.console.TableWriter.ColumnFormat.Align;
 
+/**
+ * {@link ReportClientBase}
+ *
+ * @author <a href="mailto:vitali.sjablow@open-xchange.com">Vitali Sjablow</a>
+ * @since v7.8.0
+ */
 public class ReportClientBase extends AbstractJMXTools {
 
     protected static final String CSV_NOT_SUPPORTED_MSG = "CSV support for appsuite report style not available. Please execute again with additional parameter '-o'.";
@@ -94,6 +98,8 @@ public class ReportClientBase extends AbstractJMXTools {
 
     protected static final String NO_REPORT_FOUND_MSG = "No report found. Please generate a report first by using parameter -e or -x or having a look into the CLT help (-h)!";
 
+    protected static final String NO_START_DATE_SET = "You can not run a report with an end date and no start date set!";
+
     private static final char OPT_SEND_ONLY_SHORT = 's';
 
     private static final String OPT_SEND_ONLY_LONG = "sendonly";
@@ -144,6 +150,30 @@ public class ReportClientBase extends AbstractJMXTools {
 
     private static final String OPT_RUN_AND_DELIVER_OLD_REPORT_LONG = "run-and-deliver-old-report";
 
+    private static final char OPT_APPSUITE_SET_TIMEFRAME_START_SHORT = 'S';
+
+    private static final String OPT_APPSUITE_SET_TIMEFRAME_START_LONG = "timeframe-start";
+
+    private static final char OPT_APPSUITE_SET_TIMEFRAME_END_SHORT = 'E';
+
+    private static final String OPT_APPSUITE_SET_TIMEFRAME_END_LONG = "timeframe-end";
+
+    private static final char OPT_OXAAS_SET_SINGLE_BRAND_SHORT = 'R';
+
+    private static final String OPT_OXAAS_SET_SINGLE_BRAND_LONG = "single-tenant";
+
+    private static final char OPT_OXAAS_SET_IGNORE_ADMINS_SHORT = 'A';
+
+    private static final String OPT_OXAAS_SET_IGNORE_ADMINS_LONG = "ignore-admins";
+
+    private static final char OPT_OXAAS_SET_DRIVE_METRICS_SHORT = 'D';
+
+    private static final String OPT_OXAAS_SET_DRIVE_METRICS_LONG = "drive-metrics";
+
+    private static final char OPT_OXAAS_SET_MAIL_METRICS_SHORT = 'M';
+
+    private static final String OPT_OXAAS_SET_MAIL_METRICS_LONG = "mail-metrics";
+
     private CLIOption displayonly = null;
 
     private CLIOption sendonly = null;
@@ -166,6 +196,10 @@ public class ReportClientBase extends AbstractJMXTools {
 
     private CLIOption getAsReport = null;
 
+    private CLIOption timeframeStart = null;
+
+    private CLIOption timeframeEnd = null;
+
     // After changing to appsuite report as default -x does nothing. To be backward compatible -x will still be accepted.
     private CLIOption runAndDeliverAsReport = null;
 
@@ -173,6 +207,15 @@ public class ReportClientBase extends AbstractJMXTools {
 
     private CLIOption asReportType = null;
 
+    // OXAAS options
+    private CLIOption singleTenant = null;
+
+    private CLIOption ignoreAdmins = null;
+
+    private CLIOption driveMetrics = null;
+
+    private CLIOption mailMetrics = null;
+
     public enum ReportMode {
         SENDONLY, DISPLAYONLY, SAVEONLY, MULTIPLE, DISPLAYANDSEND, NONE
     };
@@ -186,6 +229,7 @@ public class ReportClientBase extends AbstractJMXTools {
                 return;
             }
 
+            //determine the report mode
             ReportMode mode = ReportMode.NONE;
             boolean savereport = false;
 
@@ -213,102 +257,134 @@ public class ReportClientBase extends AbstractJMXTools {
                 }
             }
 
+            // Does this report have a custom timeframe, if not, start- and end-date will be the same
+            SimpleDateFormat formatter = new SimpleDateFormat("dd.MM.yyyy");
+            boolean isCustomTimeframe = false;
+            Date timeframeStart = new Date();
+            Date timeframeEnd = new Date();
+            if (parser.getOptionValue(this.timeframeStart) != null && mode != ReportMode.MULTIPLE) {
+                timeframeStart = formatter.parse((String) parser.getOptionValue(this.timeframeStart));
+                isCustomTimeframe = true;
+                if (parser.getOptionValue(this.timeframeEnd) != null) {
+                    timeframeEnd = formatter.parse((String) parser.getOptionValue(this.timeframeEnd));
+                }
+            }
+
+            // If report-type is oxaas-extended, look for other report specific options
+            String reportType = (String) parser.getOptionValue(this.asReportType);
+            if (reportType == null) {
+                reportType = "default";
+            }
+            boolean isSingleTenant = false;
+            Long singeTenantId = 0l;
+            boolean isIgnoreAdmin = false;
+            boolean isShowDriveMetrics = false;
+            boolean isShowMailMetrics = false;
+            if (reportType.equals("oxaas-extended")) {
+                if (parser.getOptionValue(singleTenant) != null) {
+                    isSingleTenant = true;
+                    singeTenantId = Long.parseLong(((String) parser.getOptionValue(singleTenant)));
+                }
+                if (parser.getOptionValue(ignoreAdmins) != null) {
+                    isIgnoreAdmin = true;
+                }
+                if (parser.getOptionValue(driveMetrics) != null) {
+                    isShowDriveMetrics = true;
+                }
+                if (parser.getOptionValue(mailMetrics) != null) {
+                    isShowMailMetrics = true;
+                }
+            }
+
+            ReportConfigs reportConfigs = new ReportConfigs(reportType, false, isCustomTimeframe, timeframeStart.getTime(), timeframeEnd.getTime(), isSingleTenant, singeTenantId, isIgnoreAdmin, isShowDriveMetrics, isShowMailMetrics);
+
+            //Start the report generation
             System.out.println("Starting the Open-Xchange report client. Note that the report generation may take a little while.");
             final MBeanServerConnection initConnection = initConnection(env);
 
-            // Is one of the appsuite report options set? In that case do something completely different.
+            // Run the new report style
             if (null == parser.getOptionValue(this.runAndDeliverOldReport)) {
-                boolean isAvailable = checkMBeanAvailability(initConnection);
-                if (isAvailable) {
-                    if (null != parser.getOptionValue(this.csv)) {
-                        System.out.println(CSV_NOT_SUPPORTED_MSG);
-                    }
-                    if (null != parser.getOptionValue(this.advancedreport)) {
-                        System.out.println(ADVANCED_NOT_SUPPORTED_MSG);
-                    }
-                    if (null != parser.getOptionValue(this.runAsReport)) {
-                        runASReport(parser.getOptionValue(this.asReportType), initConnection);
-                        inspectASReports(parser.getOptionValue(this.asReportType), initConnection);
-                        return;
-                    } else if (null != parser.getOptionValue(this.inspectAsReports)) {
-                        inspectASReports(parser.getOptionValue(this.asReportType), initConnection);
-                        return;
-                    } else if (null != parser.getOptionValue(this.cancelAsReports)) {
-                        cancelASReports(parser.getOptionValue(this.asReportType), initConnection);
-                        return;
-                    } else if (null != parser.getOptionValue(this.getAsReport)) {
-                        getASReport(parser.getOptionValue(this.asReportType), mode, savereport, initConnection);
-                        return;
-                    } else {
-                        // run and deliver AS report is no default
-                        runAndDeliverASReport(parser.getOptionValue(this.asReportType), mode, null != parser.getOptionValue(this.asReportType), savereport, initConnection);
-                        return;
-                    }
+                // the new report style does not support csv or advanced report option
+                if (null != parser.getOptionValue(this.csv)) {
+                    System.out.println(CSV_NOT_SUPPORTED_MSG);
+                }
+                if (null != parser.getOptionValue(this.advancedreport)) {
+                    System.out.println(ADVANCED_NOT_SUPPORTED_MSG);
+                }
+                if (null != parser.getOptionValue(this.runAsReport)) {
+                    runASReport(reportType, initConnection, reportConfigs);
+                    inspectASReports(reportType, initConnection);
+                    return;
+                } else if (null != parser.getOptionValue(this.inspectAsReports)) {
+                    inspectASReports(reportType, initConnection);
+                    return;
+                } else if (null != parser.getOptionValue(this.cancelAsReports)) {
+                    cancelASReports(reportType, initConnection);
+                    return;
+                } else if (null != parser.getOptionValue(this.getAsReport)) {
+                    getASReport(reportType, mode, savereport, initConnection);
+                    return;
                 } else {
-                    System.out.println("Appsuite report MBean not found! Execution continues with old style report in 'display-and-send' mode (-d -s). To run the old style report with your desired configuration use '-o' parameter or have a look at CLTs help ('-h').");
-                    mode = ReportMode.DISPLAYANDSEND;
+                    // run and deliver AS report is no default
+                    runAndDeliverASReport(mode, false, savereport, initConnection, reportConfigs);
+                    return;
                 }
             }
 
             // ... otherwise old report style
-            handleOldReport(parser, mode, savereport, initConnection);
+
+            final List<Total> totals = ObjectHandler.getTotalObjects(initConnection);
+            List<ContextDetail> contextDetails = null;
+            if (null != parser.getOptionValue(this.advancedreport)) {
+                contextDetails = ObjectHandler.getDetailObjects(initConnection);
+            }
+            List<MacDetail> macDetails = ObjectHandler.getMacObjects(initConnection);
+            final String[] versions = VersionHandler.getServerVersion();
+            Map<String, String> serverConfiguration = ObjectHandler.getServerConfiguration(initConnection);
+            final ClientLoginCount clc = ObjectHandler.getClientLoginCount(initConnection);
+            final ClientLoginCount clcYear = ObjectHandler.getClientLoginCount(initConnection, true);
+
+            switch (mode) {
+                case SENDONLY:
+                case SAVEONLY:
+                    createTransportHandler().sendReport(totals, macDetails, contextDetails, serverConfiguration, versions, clc, clcYear, savereport);
+                    break;
+
+                case DISPLAYONLY:
+                    print(totals, contextDetails, macDetails, serverConfiguration, versions, parser, clc, clcYear);
+                    break;
+
+                case NONE:
+                    System.out.println(NO_OPTION_SELECTED_USING_THE_DEFAULT_DISPLAY_AND_SEND);
+                case MULTIPLE:
+                    if (ReportMode.NONE != mode) {
+                        System.out.println(TOO_MANY_ARGUMENTS_USING_THE_DEFAULT_DISPLAY_AND_SEND);
+                    }
+                case DISPLAYANDSEND:
+                default:
+                    savereport = false;
+                    createTransportHandler().sendReport(totals, macDetails, contextDetails, serverConfiguration, versions, clc, clcYear, savereport);
+                    print(totals, contextDetails, macDetails, serverConfiguration, versions, parser, clc, clcYear);
+                    break;
+            }
+        } catch (final MalformedObjectNameException e) {
+            printServerException(e, parser);
+            sysexit(1);
+        } catch (final NullPointerException e) {
+            printServerException(e, parser);
+            sysexit(1);
+        } catch (final JSONException e) {
+            printServerException(e, parser);
+            sysexit(1);
+        } catch (final InvalidAttributeValueException e) {
+            printServerException(e, parser);
+            sysexit(1);
         } catch (final Exception e) {
             printServerException(e, parser);
             sysexit(1);
         }
     }
 
-    private boolean checkMBeanAvailability(final MBeanServerConnection initConnection) throws MBeanException, ReflectionException, IOException {
-        // check if hazelcast is enabled
-        try {
-            initConnection.invoke(getAppSuiteReportingName(), "retrieveLastReport", new Object[] { "default" }, new String[] { String.class.getCanonicalName() });
-        } catch (InstanceNotFoundException e) { // AS Report MBean not registered ==> try to run old style report
-            return false;
-        } catch (Exception e) { // Another error occurred. ==> try to run old style report
-            System.out.println("An unexpected error occurred: " + e.getMessage() + "\n");
-            System.out.println(e.getStackTrace());
-            return false;
-        }
-        return true;
-    }
-
-    private void handleOldReport(final AdminParser parser, ReportMode mode, boolean savereport, final MBeanServerConnection initConnection) throws AttributeNotFoundException, InstanceNotFoundException, MBeanException, ReflectionException, IOException, MalformedObjectNameException, InvalidAttributeValueException, JSONException {
-        final List<Total> totals = ObjectHandler.getTotalObjects(initConnection);
-        List<ContextDetail> contextDetails = null;
-        if (null != parser.getOptionValue(this.advancedreport)) {
-            contextDetails = ObjectHandler.getDetailObjects(initConnection);
-        }
-        List<MacDetail> macDetails = ObjectHandler.getMacObjects(initConnection);
-        final String[] versions = VersionHandler.getServerVersion();
-        Map<String, String> serverConfiguration = ObjectHandler.getServerConfiguration(initConnection);
-        final ClientLoginCount clc = ObjectHandler.getClientLoginCount(initConnection);
-        final ClientLoginCount clcYear = ObjectHandler.getClientLoginCount(initConnection, true);
-
-        switch (mode) {
-            case SENDONLY:
-            case SAVEONLY:
-                createTransportHandler().sendReport(totals, macDetails, contextDetails, serverConfiguration, versions, clc, clcYear, savereport);
-                break;
-
-            case DISPLAYONLY:
-                print(totals, contextDetails, macDetails, serverConfiguration, versions, parser, clc, clcYear);
-                break;
-
-            case NONE:
-                System.out.println(NO_OPTION_SELECTED_USING_THE_DEFAULT_DISPLAY_AND_SEND);
-            case MULTIPLE:
-                if (ReportMode.NONE != mode) {
-                    System.out.println(TOO_MANY_ARGUMENTS_USING_THE_DEFAULT_DISPLAY_AND_SEND);
-                }
-            case DISPLAYANDSEND:
-            default:
-                savereport = false;
-                createTransportHandler().sendReport(totals, macDetails, contextDetails, serverConfiguration, versions, clc, clcYear, savereport);
-                print(totals, contextDetails, macDetails, serverConfiguration, versions, parser, clc, clcYear);
-                break;
-        }
-    }
-
     protected TransportHandler createTransportHandler() {
         return new TransportHandler();
     }
@@ -316,15 +392,20 @@ public class ReportClientBase extends AbstractJMXTools {
     @Override
     public void setFurtherOptions(final AdminParser parser) {
         this.sendonly = setShortLongOpt(parser, OPT_SEND_ONLY_SHORT, OPT_SEND_ONLY_LONG, "Send report without displaying it (Disables default)", false, NeededQuadState.notneeded);
+
         this.displayonly = setShortLongOpt(parser, OPT_DISPLAY_ONLY_SHORT, OPT_DISPLAY_ONLY_LONG, "Display report without sending it (Disables default)", false, NeededQuadState.notneeded);
+
         this.csv = setShortLongOpt(parser, OPT_CSV_SHORT, OPT_CSV_LONG, "Show output as CSV", false, NeededQuadState.notneeded);
+
         this.advancedreport = setShortLongOpt(parser, OPT_ADVANCEDREPORT_SHORT, OPT_ADVANCEDREPORT_LONG, "Run an advanced report (could take some time with a lot of contexts)", false, NeededQuadState.notneeded);
+
         this.savereport = setShortLongOpt(parser, OPT_SAVEREPORT_SHORT, OPT_SAVEREPORT_LONG, "Save the report as JSON String instead of sending it", false, NeededQuadState.notneeded);
+
         this.showcombi = setShortLongOpt(parser, OPT_SHOWCOMBINATION_SHORT, OPT_SHOWCOMBINATION_LONG, "Show access combination for bitmask", true, NeededQuadState.notneeded);
 
         this.runAsReport = setShortLongOpt(parser, OPT_APPSUITE_RUN_REPORT_SHORT, OPT_APPSUITE_RUN_REPORT_LONG, "Schedule an appsuite style report. Will print out the reports UUID or, if a report is being generated, the UUID of the pending report", false, NeededQuadState.notneeded);
 
-        this.asReportType = setShortLongOpt(parser, OPT_APPSUITE_REPORT_TYPE_SHORT, OPT_APPSUITE_REPORT_TYPE_LONG, "The type of the report to run. Leave this off for the 'default' report.", true, NeededQuadState.notneeded);
+        this.asReportType = setShortLongOpt(parser, OPT_APPSUITE_REPORT_TYPE_SHORT, OPT_APPSUITE_REPORT_TYPE_LONG, "The type of the report to run. Leave this off for the 'default' report. 'Known reports next to 'default': 'extended', 'oscs-extended' Enables additional options, as listed below (provisioning-bundels needed)", true, NeededQuadState.notneeded);
 
         this.inspectAsReports = setLongOpt(parser, OPT_APPSUITE_INSPECT_REPORTS_LONG, "Prints information about currently running reports", false, false);
 
@@ -335,6 +416,18 @@ public class ReportClientBase extends AbstractJMXTools {
         this.runAndDeliverAsReport = setShortLongOpt(parser, OPT_APPSUITE_RUN_AND_DELIVER_REPORT_SHORT, OPT_APPSUITE_RUN_AND_DELIVER_REPORT_LONG, "Create a new report and send it immediately. Note: This command will run until the report is finished, and that could take a while. Can (and should) be combined with the options for sending, displaying or saving the report ", false, NeededQuadState.notneeded);
 
         this.runAndDeliverOldReport = setShortLongOpt(parser, OPT_RUN_AND_DELIVER_OLD_REPORT_SHORT, OPT_RUN_AND_DELIVER_OLD_REPORT_LONG, "Run old report type. Used to have a backward compatibility.", false, NeededQuadState.notneeded);
+
+        this.timeframeStart = setShortLongOpt(parser, OPT_APPSUITE_SET_TIMEFRAME_START_SHORT, OPT_APPSUITE_SET_TIMEFRAME_START_LONG, "Set the starting date of the timeframe in format: dd.mm.yyyy", true, NeededQuadState.notneeded);
+
+        this.timeframeEnd = setShortLongOpt(parser, OPT_APPSUITE_SET_TIMEFRAME_END_SHORT, OPT_APPSUITE_SET_TIMEFRAME_END_LONG, "Set the ending date of the timeframe in format: dd.mm.yyyy. If start date is set and this parameter not, the current Date is taken as timeframe end.", true, NeededQuadState.notneeded);
+
+        this.singleTenant = setShortLongOpt(parser, OPT_OXAAS_SET_SINGLE_BRAND_SHORT, OPT_OXAAS_SET_SINGLE_BRAND_LONG, "OXAAS only: Run the report for a single brand, identified by the sid of the brands admin. oxaas-extended report-type only", true, NeededQuadState.notneeded);
+
+        this.ignoreAdmins = setShortLongOpt(parser, OPT_OXAAS_SET_IGNORE_ADMINS_SHORT, OPT_OXAAS_SET_IGNORE_ADMINS_LONG, "OXAAS only: Ignore admins and dont show users of that category. oxaas-extended report-type only", false, NeededQuadState.notneeded);
+
+        this.driveMetrics = setShortLongOpt(parser, OPT_OXAAS_SET_DRIVE_METRICS_SHORT, OPT_OXAAS_SET_DRIVE_METRICS_LONG, "OXAAS only: Get drive metrics for each user. oxaas-extended report-type only", false, NeededQuadState.notneeded);
+
+        this.mailMetrics = setShortLongOpt(parser, OPT_OXAAS_SET_MAIL_METRICS_SHORT, OPT_OXAAS_SET_MAIL_METRICS_LONG, "OXAAS only: Get mail metrics for each user. oxaas-extended report-type only", false, NeededQuadState.notneeded);
     }
 
     protected void print(final List<Total> totals, final List<ContextDetail> contextDetails, final List<MacDetail> macDetails, Map<String, String> serverConfiguration, final String[] versions, final AdminParser parser, final ClientLoginCount clc, final ClientLoginCount clcYear) {
@@ -479,6 +572,15 @@ public class ReportClientBase extends AbstractJMXTools {
         return null;
     }
 
+    /**
+     * Retrieve a last report of the given type and determine further actions depending on the given {@link ReportMode}
+     * what should be done with the report.
+     * 
+     * @param reportType, the report type to retrieve
+     * @param mode, the report mode
+     * @param savereport, save this report
+     * @param server, server connection used to invoke report getting methods
+     */
     protected void getASReport(Object reportType, ReportMode mode, boolean savereport, MBeanServerConnection server) {
         if (reportType == null) {
             reportType = "default";
@@ -487,8 +589,27 @@ public class ReportClientBase extends AbstractJMXTools {
             CompositeData report = (CompositeData) server.invoke(getAppSuiteReportingName(), "retrieveLastReport", new Object[] { reportType }, new String[] { String.class.getCanonicalName() });
 
             if (report == null) {
+                CompositeData errorReport = (CompositeData) server.invoke(getAppSuiteReportingName(), "retrieveLastErrorReport", new Object[] { reportType }, new String[] { String.class.getCanonicalName() });
                 System.out.println();
-                System.out.println(NO_REPORT_FOUND_MSG);
+                if (errorReport == null) { // no successful and no error report available
+                    System.out.println(NO_REPORT_FOUND_MSG);
+                } else {
+                    String uuid = (String) errorReport.get("uuid");
+                    String type = (String) errorReport.get("type");
+                    System.out.println("The following errors have been found for the report with uuid " + uuid + " for type " + type);
+
+                    String error = (new JSONObject((String) errorReport.get("data")).get("error")).toString();
+                    JSONObject errors = new JSONObject(error.trim());
+
+                    Iterator<?> keys = errors.keys();
+                    while (keys.hasNext()) {
+                        String key = (String) keys.next();
+                        Object object = (String) errors.get(key);
+                        if (object instanceof String) {
+                            System.out.println(errors.get(key));
+                        }
+                    }
+                }
                 return;
             }
             System.out.println("");
@@ -513,7 +634,9 @@ public class ReportClientBase extends AbstractJMXTools {
                 case DISPLAYANDSEND:
                 default:
                     savereport = false;
+                    // send via TransportHandler 
                     createTransportHandler().sendASReport(report, savereport);
+                    // print to console
                     printASReport(report);
                     break;
             }
@@ -523,19 +646,51 @@ public class ReportClientBase extends AbstractJMXTools {
         }
     }
 
-    private void runAndDeliverASReport(Object reportType, ReportMode mode, boolean silent, boolean savereport, MBeanServerConnection server) {
-        if (reportType == null) {
-            reportType = "default";
-        }
+    /**
+     * Generate a report with the given parameters and wait until its finished.
+     * 
+     * @param reportType, the report type
+     * @param mode, the mode of report generation
+     * @param silent, should the current status of the report be displayed on the console
+     * @param savereport, save the report to disc or not
+     * @param server, the server connection
+     * @param isCustomTimerange, does this report have a custom timerange
+     * @param start, starting date of the report
+     * @param end, end date of the report
+     * @param isShowSingleTenant, should a report be generated for only one tenant
+     * @param singleTenantId, the id of the tenants admin
+     * @param isIgnoreAdmin, should admin users be ignored
+     * @param isShowDriveMetrics, calculate drive metrics
+     * @param isShowMailMetrics, calculate mail metrics
+     */
+    private void runAndDeliverASReport(ReportMode mode, boolean silent, boolean savereport, MBeanServerConnection server, ReportConfigs reportConfig) {
+
         try {
-            String uuid = (String) server.invoke(getAppSuiteReportingName(), "run", new Object[] { reportType }, new String[] { String.class.getCanonicalName() });
+            String uuid = "";
+            //            if (reportType.equals("oxaas-extended")) {
+            //                uuid = (String) server.invoke(getAppSuiteReportingName(), "run", new Object[] { reportType, start, end, isCustomTimerange, isShowSingleTenant, singleTenantId, isIgnoreAdmin, isShowDriveMetrics, isShowMailMetrics }, new String[] { String.class.getCanonicalName(), Date.class.getCanonicalName(), Date.class.getCanonicalName(), Boolean.class.getCanonicalName(), Boolean.class.getCanonicalName(), Long.class.getCanonicalName(), Boolean.class.getCanonicalName(), Boolean.class.getCanonicalName(), Boolean.class.getCanonicalName() });
+            //                if (uuid == null && isShowSingleTenant) {
+            //                    System.out.println("No contexts for this brand or the sid is invalid. Report generation aborted.");
+            //                    return;
+            //                }
+            //            } else {
+            //                if (isCustomTimerange) {
+            //                    uuid = (String) server.invoke(getAppSuiteReportingName(), "run", new Object[] { reportType, start, end }, new String[] { String.class.getCanonicalName(), Date.class.getCanonicalName(), Date.class.getCanonicalName() });
+            //                } else {
+            //                    
+            //                }
+            //            }
+            uuid = (String) server.invoke(getAppSuiteReportingName(), "run", new Object[] { reportConfig }, new String[] { CompositeData.class.getCanonicalName() });
 
             // Start polling
             boolean done = false;
             int charNum = 0;
 
             while (!done) {
-                CompositeData[] reports = (CompositeData[]) server.invoke(getAppSuiteReportingName(), "retrievePendingReports", new Object[] { reportType }, new String[] { String.class.getCanonicalName() });
+                CompositeData[] reports = (CompositeData[]) server.invoke(getAppSuiteReportingName(), "retrievePendingReports", new Object[] { reportConfig.getType() }, new String[] { String.class.getCanonicalName() });
+                //                if ((reports == null) || (reports.length == 0)) {
+                //                    Object o = server.invoke(getAppSuiteReportingName(), "retrieveLastErrorReport", new Object[] { reportType }, new String[] { String.class.getCanonicalName() });
+                //                }
 
                 boolean found = false;
                 for (CompositeData report : reports) {
@@ -550,18 +705,30 @@ public class ReportClientBase extends AbstractJMXTools {
 
                 done = !found;
 
+                // wait until the report is done
                 if (!done) {
                     Thread.sleep(silent ? 60000 : 10000);
                 }
             }
-
-            getASReport(reportType, mode, savereport, server);
         } catch (Exception e) {
             e.printStackTrace();
             System.exit(-1);
         }
+
+        // retrieve the latest report of this type and do what the mode tells to do
+        getASReport(reportConfig.getType(), mode, savereport, server);
     }
 
+    /**
+     * Print the status of the current report:
+     * 
+     * Example:
+     * Time | uuid | context/totalContexts | percent | how long the report has been running
+     * 13:58:43, 058b5dafb1ec4911917c24290b04b97b: 1/1 (100,00 %) ETA: 0 milliseconds
+     * 
+     * @param report
+     * @return
+     */
     private int printStatusLine(CompositeData report) {
         Long start = (Long) report.get("startTime");
         Long now = System.currentTimeMillis();
@@ -579,7 +746,7 @@ public class ReportClientBase extends AbstractJMXTools {
         }
 
         StringBuilder b = new StringBuilder();
-
+        b.append(getCurrentTimeStamp() + ", ");
         b.append(report.get("uuid")).append(": ");
         b.append(String.format("%d/%d (%.2f %%) ", finishedContexts, totalContexts, ((float) finishedContexts / totalContexts) * 100f));
         if (timePerContext > 0) {
@@ -591,6 +758,13 @@ public class ReportClientBase extends AbstractJMXTools {
         return b.length();
     }
 
+    private String getCurrentTimeStamp() {
+        SimpleDateFormat sdfDate = new SimpleDateFormat("HH:mm:ss");//dd/MM/yyyy
+        Date now = new Date();
+        String strDate = sdfDate.format(now);
+        return strDate;
+    }
+
     private void eraseStatusLine(int charNum) {
         StringBuilder b = new StringBuilder(charNum + 1);
         b.append("\n\b");
@@ -600,6 +774,12 @@ public class ReportClientBase extends AbstractJMXTools {
         System.out.print(b);
     }
 
+    /**
+     * Cancel a report of the given type.
+     * 
+     * @param reportType, the type of the report
+     * @param server, the server connection
+     */
     private void cancelASReports(Object reportType, MBeanServerConnection server) {
         if (reportType == null) {
             reportType = "default";
@@ -622,6 +802,12 @@ public class ReportClientBase extends AbstractJMXTools {
         }
     }
 
+    /**
+     * Get the status of the current running report of the given type.
+     * 
+     * @param reportType, the type of the report
+     * @param server, the server connection
+     */
     private void inspectASReports(Object reportType, MBeanServerConnection server) {
         if (reportType == null) {
             reportType = "default";
@@ -641,18 +827,34 @@ public class ReportClientBase extends AbstractJMXTools {
         }
     }
 
-    private void runASReport(Object reportType, MBeanServerConnection server) {
+    /**
+     * Run a report with the given report type and print out the uuid only. Additionally a custom timerange can be set
+     * 
+     * @param reportType, the type of the report
+     * @param server, the server connection
+     * @param isCustomTimerange, should this report have a custom timerange
+     * @param start, starting date of the timerange
+     * @param end, ending time of the timerange
+     */
+    private void runASReport(Object reportType, MBeanServerConnection server, ReportConfigs reportConfig) {
         if (reportType == null) {
             reportType = "default";
         }
         try {
-            System.out.println("\nRunning report with uuid: " + server.invoke(getAppSuiteReportingName(), "run", new Object[] { reportType }, new String[] { String.class.getCanonicalName() }));
+            String uuid = "";
+            uuid = (String) server.invoke(getAppSuiteReportingName(), "run", new Object[] { reportConfig }, new String[] { CompositeData.class.getCanonicalName() });
+            System.out.println("\nRunning report with uuid: " + uuid);
         } catch (Exception e) {
             e.printStackTrace();
             System.exit(-1);
         }
     }
 
+    /**
+     * Print a given reports header with probable time left.
+     * 
+     * @param compositeData
+     */
     private void printASDiagnostics(CompositeData compositeData) {
         Long start = (Long) compositeData.get("startTime");
         Long now = System.currentTimeMillis();
@@ -680,6 +882,11 @@ public class ReportClientBase extends AbstractJMXTools {
         }
     }
 
+    /**
+     * Print the given report to console.
+     * 
+     * @param report
+     */
     private void printASReport(CompositeData report) {
         try {
             Long start = (Long) report.get("startTime");
@@ -708,6 +915,15 @@ public class ReportClientBase extends AbstractJMXTools {
         }
     }
 
+    /**
+     * Convert milliseconds to a pretty time output String.
+     * 
+     * Example:
+     * 0 hours, 28 minutes, 3 seconds
+     * 
+     * @param interval, the time in milliseconds
+     *            @return, a pretty time String
+     */
     private String prettyPrintTimeInterval(long interval) {
         // FROM: http://stackoverflow.com/questions/635935/how-can-i-calculate-a-time-span-in-java-and-format-the-output
 
